export interface Point {
    x: number;
    y: number;
}
export declare type Drawer = (ctx: CanvasRenderingContext2D, p: Point) => void;
export interface ImagesOptions {
    width?: 'original' | number;
    height?: 'original' | number;
    rotate?: 'auto' | number;
    minInterval?: number;
    borderColor?: string;
    borderWidth?: number;
}
export interface LineOptions {
    width?: number;
    strokeColor?: string;
    lineDash?: number[];
}
export interface RectsOptions {
    width?: number;
    height?: number;
    rotate?: 'auto' | number;
    minInterval?: number;
    strokeColor?: string;
    strokeWidth?: number;
    fillColor?: string;
}
export interface CirclesOptions {
    radius?: number;
    minInterval?: number;
    strokeColor?: string;
    strokeWidth?: number;
    fillColor?: string;
}
export interface TrianglesOptions {
    edgeLength?: number;
    rotate?: 'auto' | number;
    minInterval?: number;
    strokeColor?: string;
    strokeWidth?: number;
    fillColor?: string;
}
export interface CrossMarksOptions {
    lineLength?: number;
    rotate?: 'auto' | number;
    minInterval?: number;
    strokeColor?: string;
    strokeWidth?: number;
}
export interface WithDrawerOptions {
    minInterval?: number;
}
export default class DrawBetween {
    private ctx;
    constructor(elem: HTMLElement);
    static appendCanvas(elem: HTMLElement): HTMLCanvasElement;
    static getPointsBetween(p0: Point, p1: Point, minInterval: number): Point[];
    static getPointsFor(p0: Point, p1: Point, width: number, height: number, minInterval?: number): Point[];
    static getRotateFor(p0: Point, p1: Point): number;
    clear(): void;
    images(p0: Point, p1: Point, imageUrl: string, options?: ImagesOptions): void;
    line(p0: Point, p1: Point, options?: LineOptions): void;
    circles(p0: Point, p1: Point, options?: CirclesOptions): void;
    rects(p0: Point, p1: Point, options?: RectsOptions): void;
    triangles(p0: Point, p1: Point, options?: TrianglesOptions): void;
    crossMarks(p0: Point, p1: Point, options?: CrossMarksOptions): void;
    withDrawer(p0: Point, p1: Point, drawer: Drawer, options?: WithDrawerOptions): void;
}
