/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

// drawbetween/index.js
var _ImagesOptions = /** @class */ (function () {
    function _ImagesOptions() {
        this.width = 'original';
        this.height = 'original';
        this.rotate = 'auto';
        this.minInterval = 0;
        this.borderColor = "#000";
        this.borderWidth = 0;
    }
    return _ImagesOptions;
}());
var _LineOptions = /** @class */ (function () {
    function _LineOptions() {
        this.width = 1;
        this.strokeColor = "#000000";
        this.lineDash = [0, 0];
    }
    return _LineOptions;
}());
var _RectsOptions = /** @class */ (function () {
    function _RectsOptions() {
        this.width = 20;
        this.height = 20;
        this.rotate = 'auto';
        this.minInterval = 0;
        this.strokeColor = "#000";
        this.strokeWidth = 1;
        this.fillColor = "";
    }
    return _RectsOptions;
}());
var _CirclesOptions = /** @class */ (function () {
    function _CirclesOptions() {
        this.radius = 10;
        this.minInterval = 0;
        this.strokeColor = "#000";
        this.strokeWidth = 1;
        this.fillColor = "";
    }
    return _CirclesOptions;
}());
var _TrianglesOptions = /** @class */ (function () {
    function _TrianglesOptions() {
        this.edgeLength = 20;
        this.rotate = 'auto';
        this.minInterval = 0;
        this.strokeColor = "#000";
        this.strokeWidth = 1;
        this.fillColor = "";
    }
    return _TrianglesOptions;
}());
var _CrossMarksOptions = /** @class */ (function () {
    function _CrossMarksOptions() {
        this.lineLength = 20;
        this.rotate = 'auto';
        this.minInterval = 0;
        this.strokeColor = "#000";
        this.strokeWidth = 1;
    }
    return _CrossMarksOptions;
}());
var _WithDrawerOptions = /** @class */ (function () {
    function _WithDrawerOptions() {
        this.minInterval = 20;
    }
    return _WithDrawerOptions;
}());
//
// DrawBetween
//
var DrawBetween = /** @class */ (function () {
    function DrawBetween(elem) {
        var cv = DrawBetween.appendCanvas(elem);
        this.ctx = cv.getContext("2d");
    }
    // appendCanvas()
    DrawBetween.appendCanvas = function (elem) {
        var cv = document.createElement("canvas");
        var resizeCv = function () {
            cv.width = elem.clientWidth;
            cv.height = elem.clientHeight;
        };
        cv.style.pointerEvents = "none";
        resizeCv();
        // make canvas responsive:
        // TODO: actually it is not enough to just listen to
        //       window.resize event. We should listen to
        //       the parent element's resize event...
        //       Use 'ResizeObserver' ? (available only in Chrome).
        window.addEventListener("resize", resizeCv);
        elem.appendChild(cv);
        return cv;
    };
    // getPointsBetween()
    //  return points between (p0, p1) with specified intervals.
    DrawBetween.getPointsBetween = function (p0, p1, minInterval) {
        // len: length of (p0->p1)
        var len = Math.sqrt(Math.pow(p0.x - p1.x, 2) + Math.pow(p0.y - p1.y, 2));
        // n: number of points
        if (minInterval < 1) {
            return [];
        }
        var _n = Math.floor(len / minInterval);
        var remainder = len - _n * minInterval;
        var n = _n + 1;
        // interval:
        var interval = minInterval + remainder / n;
        // dx, dy: amount of change on X and Y to l on the Line(y=mx).
        var l = interval;
        var dx;
        var dy;
        if (p0.x === p1.x) {
            // line(p0->p1): x = C
            dx = 0;
            dy = l;
            if (p1.y < p0.y) {
                dy = -dy;
            }
        }
        else {
            // line(p0->p1): y = mx
            var m = (p0.y - p1.y) / (p0.x - p1.x);
            // l^2 = x^2 + y^2
            // -> l^2 = x^2 + (mx)^2
            // -> x^2 = l^2 / (m^2 + 1)
            dx = Math.sqrt(Math.pow(l, 2) / (Math.pow(m, 2) + 1));
            dy = m * dx;
            if (p1.x < p0.x) {
                dx = -dx;
                dy = -dy;
            }
        }
        var points = [];
        for (var i = 0; i < n; i++) {
            points.push({
                x: Math.floor(p0.x + dx * i),
                y: Math.floor(p0.y + dy * i)
            });
        }
        return points;
    };
    // getPointsFor()
    //   return points for drawing objects (width, height)
    //   at the specified intervals.
    DrawBetween.getPointsFor = function (p0, p1, width, height, minInterval) {
        if (minInterval === void 0) { minInterval = 0; }
        // len: object's length on line(p0->p1);
        var len;
        if (p0.x === p1.x) {
            // y = C
            len = height;
        }
        else if (p0.y === p1.y) {
            // x = C
            len = width;
        }
        else {
            // line(p0->p1): y = mx
            var m = (p0.y - p1.y) / (p0.x - p1.x);
            if (Math.abs(m) * width < height) {
                len = Math.sqrt(Math.pow(width, 2) + Math.pow(m * width, 2));
            }
            else {
                len = Math.sqrt(Math.pow(height / m, 2) + Math.pow(height, 2));
            }
        }
        return DrawBetween.getPointsBetween(p0, p1, len + minInterval);
    };
    // getRotateFor()
    DrawBetween.getRotateFor = function (p0, p1) {
        var vx = p1.x - p0.x;
        var m = Math.atan((p1.y - p0.y) / (p1.x - p0.x));
        return (0 < vx) ? (m - Math.PI / 2) : (m + Math.PI / 2);
    };
    // clear()
    DrawBetween.prototype.clear = function () {
        var cv = this.ctx.canvas;
        this.ctx.clearRect(0, 0, cv.width, cv.height);
    };
    // images()
    DrawBetween.prototype.images = function (p0, p1, imageUrl, options) {
        var _this = this;
        var defaultOptions = new _ImagesOptions();
        var opts = __assign(__assign({}, defaultOptions), options);
        var minInterval = opts.minInterval;
        var rotate = (opts.rotate === 'auto') ?
            DrawBetween.getRotateFor(p0, p1) : opts.rotate;
        var doit = function (img) {
            var imageWidth = opts.width === 'original' ?
                img.width : opts.width;
            var imageHeight = opts.height === 'original' ?
                img.height : opts.height;
            _this.ctx.save();
            if (0 < opts.borderWidth) {
                _this.ctx.lineWidth = opts.borderWidth;
                _this.ctx.strokeStyle = opts.borderColor;
            }
            DrawBetween.getPointsFor(p0, p1, imageWidth + opts.borderWidth * 2, imageWidth + opts.borderWidth * 2, minInterval).forEach(function (p) {
                _this.ctx.save();
                // change rotation center & rotate.
                _this.ctx.translate(p.x, p.y);
                _this.ctx.rotate(rotate);
                _this.ctx.beginPath();
                // draw border.
                if (0 < opts.borderWidth) {
                    _this.ctx.rect(Math.floor(opts.borderWidth / 2), Math.floor(opts.borderWidth / 2), imageWidth + opts.borderWidth, imageHeight + opts.borderWidth);
                    _this.ctx.stroke();
                }
                // draw image.
                _this.ctx.drawImage(img, opts.borderWidth, opts.borderWidth, imageWidth, imageHeight);
                // restore for next loop.
                _this.ctx.restore();
            });
            // restore for next call.
            _this.ctx.restore();
        };
        var img = new Image();
        img.onload = function () {
            doit(img);
        };
        img.src = imageUrl;
    };
    // line()
    DrawBetween.prototype.line = function (p0, p1, options) {
        var defaultOptions = new _LineOptions();
        var opts = __assign(__assign({}, defaultOptions), options);
        this.ctx.save();
        this.ctx.lineWidth = opts.width;
        this.ctx.strokeStyle = opts.strokeColor;
        this.ctx.setLineDash(opts.lineDash);
        this.ctx.beginPath();
        this.ctx.moveTo(p0.x, p0.y);
        this.ctx.lineTo(p1.x, p1.y);
        this.ctx.stroke();
        this.ctx.restore();
    };
    // circles()
    DrawBetween.prototype.circles = function (p0, p1, options) {
        var _this = this;
        var defaultOptions = new _CirclesOptions();
        var opts = __assign(__assign({}, defaultOptions), options);
        var radius = opts.radius;
        var minInterval = opts.minInterval;
        this.ctx.save();
        if (0 < opts.strokeWidth) {
            this.ctx.lineWidth = opts.strokeWidth;
            this.ctx.strokeStyle = opts.strokeColor;
        }
        if (opts.fillColor != '') {
            this.ctx.fillStyle = opts.fillColor;
        }
        DrawBetween.getPointsFor(p0, p1, radius * 2 + opts.strokeWidth, radius * 2 + opts.strokeWidth, minInterval).forEach(function (p) {
            _this.ctx.beginPath();
            _this.ctx.arc(p.x, p.y, radius, 0, 2 * Math.PI);
            if (0 < opts.strokeWidth) {
                _this.ctx.stroke();
            }
            if (opts.fillColor !== '') {
                _this.ctx.fill();
            }
        });
        this.ctx.restore();
    };
    // rects()
    DrawBetween.prototype.rects = function (p0, p1, options) {
        var _this = this;
        var defaultOptions = new _RectsOptions();
        var opts = __assign(__assign({}, defaultOptions), options);
        var width = opts.width;
        var height = opts.height;
        var rotate = (opts.rotate === 'auto') ?
            DrawBetween.getRotateFor(p0, p1) : opts.rotate;
        var minInterval = opts.minInterval;
        this.ctx.save();
        if (0 < opts.strokeWidth) {
            this.ctx.lineWidth = opts.strokeWidth;
            this.ctx.strokeStyle = opts.strokeColor;
        }
        if (opts.fillColor !== '') {
            this.ctx.fillStyle = opts.fillColor;
        }
        DrawBetween.getPointsFor(p0, p1, width + opts.strokeWidth, height + opts.strokeWidth, minInterval).forEach(function (p) {
            _this.ctx.save();
            _this.ctx.translate(p.x, p.y);
            _this.ctx.rotate(rotate);
            _this.ctx.beginPath();
            _this.ctx.rect(0, 0, width, height);
            if (0 < opts.strokeWidth) {
                _this.ctx.stroke();
            }
            if (opts.fillColor !== '') {
                _this.ctx.fill();
            }
            _this.ctx.restore();
        });
        this.ctx.restore();
    };
    // triangles()
    DrawBetween.prototype.triangles = function (p0, p1, options) {
        var _this = this;
        var defaultOptions = new _TrianglesOptions();
        var opts = __assign(__assign({}, defaultOptions), options);
        var edgeLength = opts.edgeLength;
        var dx = Math.floor(edgeLength / 2);
        var dy = Math.floor((edgeLength * Math.sqrt(3)) / 2);
        var minInterval = opts.minInterval;
        var rotate = (opts.rotate === 'auto') ?
            DrawBetween.getRotateFor(p0, p1) : opts.rotate;
        this.ctx.save();
        if (0 < opts.strokeWidth) {
            this.ctx.lineWidth = opts.strokeWidth;
            this.ctx.strokeStyle = opts.strokeColor;
        }
        if (opts.fillColor !== '') {
            this.ctx.fillStyle = opts.fillColor;
        }
        DrawBetween.getPointsFor(p0, p1, edgeLength + opts.strokeWidth, edgeLength + opts.strokeWidth, minInterval).forEach(function (p) {
            _this.ctx.save();
            _this.ctx.translate(p.x, p.y);
            _this.ctx.rotate(rotate);
            _this.ctx.beginPath();
            // top
            _this.ctx.moveTo(0, 0);
            // bottom-left
            _this.ctx.lineTo(-dx, +dy);
            // bottom-right
            _this.ctx.lineTo(+dx, +dy);
            _this.ctx.closePath();
            if (0 < opts.strokeWidth) {
                _this.ctx.stroke();
            }
            if (opts.fillColor !== '') {
                _this.ctx.fill();
            }
            _this.ctx.restore();
        });
        this.ctx.restore();
    };
    // crossMarks()
    DrawBetween.prototype.crossMarks = function (p0, p1, options) {
        var _this = this;
        var defaultOptions = new _CrossMarksOptions();
        var opts = __assign(__assign({}, defaultOptions), options);
        var lineLength = opts.lineLength;
        var d = Math.floor(lineLength / (2 * Math.sqrt(2)));
        var size = Math.floor((lineLength + opts.strokeWidth) / Math.sqrt(2));
        var minInterval = opts.minInterval;
        var rotate = (opts.rotate === 'auto') ?
            DrawBetween.getRotateFor(p0, p1) : opts.rotate;
        this.ctx.save();
        if (0 < opts.strokeWidth) {
            this.ctx.lineWidth = opts.strokeWidth;
            this.ctx.strokeStyle = opts.strokeColor;
        }
        DrawBetween.getPointsFor(p0, p1, size, size, minInterval).forEach(function (p) {
            _this.ctx.save();
            _this.ctx.translate(p.x, p.y);
            _this.ctx.rotate(rotate);
            _this.ctx.beginPath();
            _this.ctx.beginPath();
            // upper-left
            _this.ctx.moveTo(-d, -d);
            // bottom-right
            _this.ctx.lineTo(+d, +d);
            // upper-right
            _this.ctx.moveTo(+d, -d);
            // bottom-left
            _this.ctx.lineTo(-d, +d);
            if (0 < opts.strokeWidth) {
                _this.ctx.stroke();
            }
            _this.ctx.restore();
        });
        this.ctx.restore();
    };
    // withDrawer()
    DrawBetween.prototype.withDrawer = function (p0, p1, drawer, options) {
        var _this = this;
        var defaultOptions = new _WithDrawerOptions();
        var opts = __assign(__assign({}, defaultOptions), options);
        var minInterval = opts.minInterval;
        DrawBetween.getPointsFor(p0, p1, 0, 0, minInterval).forEach(function (p) {
            drawer(_this.ctx, p);
        });
    };
    return DrawBetween;
}());

export default DrawBetween;
